#!/usr/bin/perl
#
# add2jsgal.pl v0.20 - add images to gallery 
#
# Copyright (c) 2005-2008 by Andreas S. Krebs (jsgal@sberk.net)
#
# this script automatically saves a resized version of the image and a thumb of
# the image in the folders specified by the gallery xml file and finally updates
# the xml itself
#

use Getopt::Std;
use File::stat;
use Date::Format;
use XML::DOM;
use Image::Magick;

$IMAGE_SIZE="1024x768";
$IMAGE_QUAL=85;
$THUMB_SIZE="200x150";
$THUMB_QUAL=75;

$USAGE="Usage: add2jsgal.pl -g gallery [-c title -i images_path -t thumbs_path] image ...\n  if -c, -i and -t are provided a new gallery is created if it doesn't exist\n";

getopts('g:c:i:t:') or die $USAGE;

if($#ARGV < 0 || !defined $opt_g)
{
  print $USAGE;
  exit 0;
}

# first create/load the gallery
my $gallery, $imgsnode, $imagesdir, $thumbsdir, $defscale, $aspect;

if (! -e "$opt_g") 
{
  #create a new gallery if initial params are provided
  if(!defined $opt_c || !defined $opt_i || !defined $opt_t) 
  {
    print $USAGE;
    exit 0;
  }

  $imagesdir=$opt_i;
  $thumbsdir=$opt_t;
  $defscale="vert";
  $aspect=4/3;

  $gallery=XML::DOM::Document->new();
  $gallery->setXMLDecl($gallery->createXMLDecl('1.0','UTF-8')); 

  my $dt;

#  $dt=$gallery->createDocumentType("jsgal", "jsgal.dtd");
#  since createDocumentType doesn't seem to work we create the doctype manual

  $dt=XML::DOM::DocumentType->new();
  $dt->setOwnerDocument($gallery);
  $dt->setName("jsgal");
  $dt->setSysId("jsgal.dtd");
  $gallery->setDoctype($dt);
  
  my $pel, $el;

  $pel=$gallery->createElement("jsgal");
  $gallery->appendChild($pel);

  $el=$gallery->createElement("title");
  $el->appendChild($gallery->createTextNode($opt_c));
  $pel->appendChild($el);

  $el=$gallery->createElement("thumbs");
  $el->setAttribute("rows", "0");
  $el->setAttribute("columns", "0");
  $el->setAttribute("pagesinrow", "20");
  $el->setAttribute("width", "200");
  $el->setAttribute("height", "150");
  $pel->appendChild($el);

  $imgsnode=$gallery->createElement("images");
  $imgsnode->setAttribute("imagesdir", $imagesdir);
  $imgsnode->setAttribute("thumbsdir", $thumbsdir);
  $imgsnode->setAttribute("width", "800");
  $imgsnode->setAttribute("height", "600");
  $imgsnode->setAttribute("target", "null");
  $imgsnode->setAttribute("defscale", $defscale);
  $pel->appendChild($imgsnode);
}
else
{
  my $parser=new XML::DOM::Parser;

  $gallery=$parser->parsefile($opt_g);
  $imgsnode=$gallery->getElementsByTagName("images")->item(0);

  $imagesdir=$imgsnode->getAttribute("imagesdir");
  $thumbsdir=$imgsnode->getAttribute("thumbsdir");

  # now take care of converting scaling options
  $defscale=$imgsnode->getAttribute("defscale");
  
  my $thumbsnode=$gallery->getElementsByTagName("thumbs")->item(0);
  my $w=$thumbsnode->getAttribute("width");
  my $h=$thumbsnode->getAttribute("height");
  my $sc;

  if($w eq "null")
  {
    $w=($h*4)/3;	# we expect 4:3 to be the default
    $thumbsnode->setAttribute("width", $w);
  }
  elsif($h eq "null")
  {
    $h=($w*3)/4;
    $thumbsnode->setAttribute("height", $h);
  }

  $w=$imgsnode->getAttribute("width");
  $h=$imgsnode->getAttribute("height");

  if($w ne "null" && $h ne "null")
  {
    $sc="both";
  }
  elsif($w eq "null")
  {
    $w=($h*4)/3;	# we expect 4:3 to be the default
    $imgsnode->setAttribute("width", $w);
    $sc="vert";
  }
  elsif($h eq "null")
  {
    $h=($w*3)/4;
    $imgsnode->setAttribute("height", $h);
    $sc="horiz";
  }

  $aspect=$w/$h;
  if(!$defscale)
  {
    $defscale=$sc;
    $imgsnode->setAttribute("defscale", $defscale);
  }
}

#now add the new files to the gallery
foreach $file (@ARGV)
{
  my $image=new Image::Magick;
  my $x=$image->Read($file);
  
  if($x)
  {
    warn $x;
  }
  else
  {
    my $name=$file;

    # get the name of the image
    $name=~s/.*\///;
    $name=~s/.*\\//;
    $name=~s/.*://;

    # get the EXIF:DateTime if any
    my $dt=$image->Get("EXIF:DateTimeOriginal") ;
    $dt=~s/\.//;
    
    if($dt!="" && $dt!="0000:00:00 00:00:00")
    {
      $dt=~s/:/\//;
      $dt=~s/:/\//;
    }
    else
    {
      # no EXIF DateTime so we use the file date
      my $ftime=stat($file)->mtime;

      $dt=time2str("%Y/%m/%d %H:%M:%S", $ftime);
    }
 
    # new get rid of APP1 profile from camera
#    $image->Profile(name=>"APP1");
    $image->Profile(name=>"*");

    # now create the image
    scaleImage($image, $IMAGE_SIZE, $IMAGE_QUAL, "$imagesdir/$name");

    # now create the thumg
    scaleImage($image, $THUMB_SIZE, $THUMB_QUAL, "$thumbsdir/$name");

    # now add image to DOM (and also add text nodes for pretty printing - until we find a better solution)
    my $imgtag=$gallery->createElement("image");
    $imgtag->setAttribute("file", $name);

    if($dt ne "")
    {
      $imgtag->setAttribute("name", $dt);
    }

    if($defscale ne "both") # if scaling is both, we don't set it on any image based on aspect !!!
    {
      my $a=$image->Get('width') / $image->Get('height');

      if($a>$aspect)
      {
         if($defscale ne "horiz")
         {
           $imgtag->setAttribute("scale", "horiz");
         }
      }
      elsif($defscale ne "vert")
      {
        $imgtag->setAttribute("scale", "vert");
      }
    }

    $imgsnode->appendChild($gallery->createTextNode("  "));
    $imgsnode->appendChild($imgtag);
    $imgsnode->appendChild($gallery->createTextNode("\n  "));
  }

  undef $image;
}

#write gallery again
print "Updating: $opt_g\n";
$gallery->printToFile ($opt_g);
$gallery->dispose;

1;

sub scaleImage
{
  my($img,$geom,$qual,$outfile)=@_;

  my($w,$h)=split('x',$geom);
  my $imgw=$img->Get('width');
  my $imgh=$img->Get('height');

  my $x;

  if($imgw>$w || $imgh>$h)
  {
    $img=$img->Clone();

    $x=$img->Scale($geom);
    warn $x if $x;
  }
  else
  {
    $geom=$imgw."x".$imgh;
  }

  print "  Creating: $outfile ($geom)\n";
  $img->Set(quality=>$qual);
  $x=$img->Write($outfile);
  warn $x if $x;

  undef $img;
}
